from f_scheduler.modules.dag import DAG
from f_scheduler.operators import DefaultFunctionOperator
from f_scheduler.operators.condition_op import ConditionOperator
from f_scheduler.operators.iter_function_op import IterFunctionOperator
from f_scheduler.utils.converter import Converter

__all__ = ["ConditionOperator", "DefaultFunctionOperator",
           "IterFunctionOperator", "DAG", "Converter"]



class BaseOperator:
    def __init__(self, task_id: str):
        self.task_id = task_id
        self.next_task = None

    def execute(self, context: dict):
        raise NotImplementedError()

    def next(self, task):
        self.next_task = task
        return self

    def run(self):
        result = self.execute({})
        if result and self.next_task:
            self.next_task.run()
        else:
            return False



from graphlib import TopologicalSorter


class DAG:
    def __init__(self, use_graphlib=False):
        self.tasks = {}
        self.graph = TopologicalSorter()
        self.use_graphlib = use_graphlib

    def add_task(self, task):
        self.tasks[task.task_id] = task
        if self.use_graphlib:
            self.graph.add(task.task_id)

    def set_downstream(self, task_id, next_task_id):
        if not self.use_graphlib:
            task = self.tasks[task_id]
            next_task = self.tasks[next_task_id]
            task.next(next_task)
        else:
            self.graph.add(task_id, next_task_id)

    def run(self, start_task_id=None):
        if not self.use_graphlib:
            if start_task_id is None:
                start_task_id = list(self.tasks.keys())[0]
            start_task = self.tasks[start_task_id]
            start_task.run()
        else:
            order = list(self.graph.static_order())
            order.reverse()
            for task_id in order:
                self.tasks[task_id].run()

    def clear(self):
        self.tasks.clear()

    def get_return_value(self, task_id):
        return self.tasks[task_id].return_value

    def get_all_tasks(self):
        return self.tasks

    def update_task(self, task_id, new_param):
        if not self.use_graphlib:
            task = self.tasks[task_id]
        else:
            task = self.tasks.get(task_id)
        if task:
            task.param = new_param
        else:
            print(f"No task found with id: {task_id}")



import json

from f_scheduler.operators.base_op import BaseOperator


class IterFunctionOperator(BaseOperator):
    def __init__(self, function, param, iterations, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.function = function
        self.param = param
        self.iterations = iterations
        self.return_value = []

    def execute(self, context: dict) -> bool:
        try:
            for _ in range(self.iterations):
                result = self.function(*self.param)
                if result is not None:
                    self.return_value.append(result)
            return True
        except Exception as e:
            print(f"Function execution failed with error: {e}")
            return False

    def __repr__(self):
        d = {'ID': self.task_id, 'Function': self.function.__name__, 'Param': self.param}
        return json.dumps(d)



class Converter:
    def __init__(self, dag):
        self.dag = dag

    def convert_list_to_dag(self, list_of_tasks):
        for i in range(len(list_of_tasks) - 1):
            self.dag.set_downstream(list_of_tasks[i], list_of_tasks[i + 1])
        return self.dag



import json

from f_scheduler.operators.base_op import BaseOperator


class ConditionOperator(BaseOperator):
    def __init__(self, condition: bool, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.condition = condition

    def execute(self, context: dict) -> bool:
        return self.condition

    def __repr__(self):
        d = {'ID': self.task_id, 'Condition': self.condition}
        return json.dumps(d)



from f_scheduler.operators.condition_op import ConditionOperator
from f_scheduler.operators.function_op import DefaultFunctionOperator
from f_scheduler.operators.iter_function_op import IterFunctionOperator


operators = {
    'ConditionOperator': ConditionOperator,
    'FunctionOperator': DefaultFunctionOperator,
    'IterFunctionOperator': IterFunctionOperator
}
